package com.lantone.qc.kernel.catalogue.firstpagerecord;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.DeathRecordDoc;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @ClassName : FIRP02905
 * @Description : 接收机构名称未填写
 * @Author : 胡敬
 * @Date: 2020-06-04 16:14
 */
@Component
public class FIRP02905 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        if (inputInfo.getFirstPageRecordDoc() == null) {
            status.set("0");
            return;
        }
        DeathRecordDoc deathRecordDoc = inputInfo.getDeathRecordDoc();
        if (deathRecordDoc != null) {
            status.set("0");
            return;
        }
        if (inputInfo.getFirstPageRecordDoc().getStructureMap() != null) {
            Map<String, String> firstpageStructureMap = inputInfo.getFirstPageRecordDoc().getStructureMap();
            String outWay = firstpageStructureMap.get(Content.outWay);
            String name = firstpageStructureMap.get(Content.receivingOrganization);
            //医嘱离院不需要接受机构（宁海一院）
            if (StringUtil.isNotBlank(outWay) && ("医嘱离院".equals(outWay) || "死亡".equals(outWay))) {
                status.set("0");
                return;
            }
            if (!CatalogueUtil.isEmpty(name)) {
                status.set("0");
            }
        }
    }
}