package com.lantone.qc.kernel.catalogue.hospital.shengzhouyy.operationdiscussion;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.operation.OperationDiscussionDoc;
import com.lantone.qc.pub.model.doc.operation.OperationDoc;
import com.lantone.qc.pub.model.doc.operation.OperationRecordDoc;
import com.lantone.qc.pub.util.StringUtil;
import org.apache.commons.lang3.time.DateUtils;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName : OPE02930
 * @Description : 术后首次病程未即刻完成
 * @Author : 胡敬
 * @Date: 2020-06-15 17:30
 */
@Component
public class OPE02930 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        List<OperationDoc> operationDocs = inputInfo.getOperationDocs();
        if (operationDocs == null || operationDocs.size() == 0) {
            status.set("0");
            return;
        }
        for (OperationDoc operationDoc : operationDocs) {
            OperationRecordDoc operationRecordDoc = operationDoc.getOperationRecordDoc();
            OperationDiscussionDoc operationDiscussionDoc = operationDoc.getOperationDiscussionDoc();
            if (operationRecordDoc == null || operationDiscussionDoc == null) {
                continue;
            }
            Map<String, String> operationRecordStructureMap = operationRecordDoc.getStructureMap();
            Map<String, String> structureMap = operationDiscussionDoc.getStructureMap();
            String operationEndDateStr = operationRecordStructureMap.get("结束时间");
            String discussionRecordDateStr = structureMap.get("病历日期");
            if (StringUtil.isEmpty(operationEndDateStr) || StringUtil.isBlank(discussionRecordDateStr)) {
                continue;
            }
            if (StringUtil.parseDateTime(operationEndDateStr) == null
                    || StringUtil.parseDateTime(discussionRecordDateStr) == null) {
                continue;
            }
            Date operationEndDate = StringUtil.parseDateTime(operationEndDateStr);
            Date discussionRecordDate = StringUtil.parseDateTime(discussionRecordDateStr);
            if (operationEndDate.before(discussionRecordDate) || DateUtils.isSameDay(operationEndDate, discussionRecordDate)) {
                if (CatalogueUtil.compareTime(operationEndDate, discussionRecordDate, 2 * 60L)) {
                    status.set("-1");
                    return;
                }
            }
        }
    }
}