package com.lantone.qc.kernel.catalogue.hospital.taizhou.threelevelward;

import com.lantone.qc.dbanaly.util.KernelConstants;
import com.lantone.qc.dbanaly.util.SpecialStorageUtil;
import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.DeathCaseDiscussDoc;
import com.lantone.qc.pub.model.doc.DeathRecordDoc;
import com.lantone.qc.pub.model.doc.LeaveHospitalDoc;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.util.SpringContextUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;

/**
 * @ClassName : THR0144
 * @Description : 患者出院前无上级医师(主治及以上)同意出院的病程记录
 * @Author : 胡敬
 * @Date: 2020-03-19 17:20
 */
@Component
public class THR0144 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        if (inputInfo.getThreeLevelWardDocs().size() == 0) {
            status.set("0");
            return;
        }
        LeaveHospitalDoc leaveHospitalDoc = inputInfo.getLeaveHospitalDoc();
        DeathRecordDoc deathRecordDoc = inputInfo.getDeathRecordDoc();
        DeathCaseDiscussDoc deathCaseDiscussDoc = inputInfo.getDeathCaseDiscussDoc();
        /* 如果没有出院小结，就不报         如果有死亡记录、死亡讨论记录，也不报 */
        if (leaveHospitalDoc == null || deathRecordDoc != null || deathCaseDiscussDoc != null) {
            status.set("0");
            return;
        }
        List<ThreeLevelWardDoc> allDoctorWradDocs = inputInfo.getThreeLevelWardDocs().get(0).getAllDoctorWradDocs();
        if (allDoctorWradDocs.size() == 0) {
            status.set("0");
            return;
        }
        int j = 0; //最后两次查房记录
        for (int i = allDoctorWradDocs.size() - 1; i >= 0 && j < 2; i--) {
            j++;
            ThreeLevelWardDoc lastWardDoc = allDoctorWradDocs.get(i);
            Map<String, String> structureMap = lastWardDoc.getStructureMap();
            String conditionRecord = structureMap.get("病情记录");
            String treatmentPlan = structureMap.get("治疗计划和措施");
            String title = structureMap.get("查房标题");
            if (StringUtil.isBlank(title) || title.contains("病理报告")) {
                continue;
            }
            conditionRecord = StringUtil.isBlank(conditionRecord) ? "" : conditionRecord;
            treatmentPlan = StringUtil.isBlank(treatmentPlan) ? "" : treatmentPlan;
            //主任或主治查房 标题需要包含“主任”或“主治”，内容需要包含“出院”
            if ((conditionRecord.contains("出院") || treatmentPlan.contains("出院") || conditionRecord.contains("转上级医院"))
                    && (CatalogueUtil.subTitle(title).contains(Content.attend)
                    || CatalogueUtil.subTitle(title).contains(Content.director)
                    || CatalogueUtil.subTitle(title).contains("主刀"))) {
                status.set("0");
                return;
            }
            //普通查房 内容需要包含“上级”和“出院”,或者内容包含出院，记录医师为主治、主任医师
            if ((StringUtil.isBlank(title)
                    || title.contains("日常查房记录")
                    || title.contains("普通查房记录")
                    || title.contains("病程记录")
                    || title.contains("日常病程记录"))
                    &&
                    (conditionRecord.contains("出院") || treatmentPlan.contains("出院"))) {
                if (conditionRecord.contains("上级") || conditionRecord.contains("主任") || conditionRecord.contains("主治")) {
                    status.set("0");
                    return;
                }

                String doctorName = structureMap.get("记录医师");
                if (StringUtil.isNotBlank(doctorName)) {
                    String doctorTitle = getTitle(doctorName);
                    if (StringUtils.isNotEmpty(doctorTitle) && (doctorTitle.contains("主治") || doctorTitle.contains("主任"))) {
                        status.set("0");
                        return;
                    }
                }
            }
        }
    }

    private String getTitle(String doctorName) {
        String title = "";
        if (StringUtil.isBlank(doctorName)) {
            return title;
        }
        SpecialStorageUtil specialStorageUtil = SpringContextUtil.getBean("specialStorageUtil");
        Map<String, Map<String, String>> surgeon = specialStorageUtil.getJsonStringValue(KernelConstants.HOSPITAL_DOCTOR_MAP);
        if (surgeon != null) {
            Map<String, String> doctor = (Map) surgeon.get(doctorName);
            if (doctor != null) {
                title = doctor.get("professor");
            }
        }
        return title;
    }
}