package com.lantone.qc.kernel.catalogue.hospital.taizhou.threelevelward;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.kernel.util.CatalogueUtil;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.DoctorAdviceDoc;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.util.DateUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @ClassName : THR0596
 * @Description : 病危患者每天应有一次病程记录
 * @Author : 胡敬
 * @Date: 2020-03-28 14:22
 */
@Component
public class THR0596 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        if (inputInfo.getDoctorAdviceDocs().size() == 0 || inputInfo.getThreeLevelWardDocs().size() == 0) {
            return;
        }
        List<DoctorAdviceDoc> doctorAdviceDocs = inputInfo.getDoctorAdviceDocs();
        //排序后医嘱信息
        List<DoctorAdviceDoc> doctorAdviceSortDocs = doctorAdviceDocs
                .stream()
                .filter(x -> StringUtil.isNotBlank(x.getStructureMap().get("医嘱项目名称")) && x.getStructureMap().get("医嘱项目名称").contains("病危"))
                .sorted(Comparator.comparing(x -> x.getStructureMap().get("医嘱开始时间")))
                .collect(Collectors.toList());

        List<String> adviceEntry = new ArrayList<>();
        String adviceDate = "";
        for (int i = 0; i < doctorAdviceSortDocs.size(); i++) {
            String startDateStr = doctorAdviceSortDocs.get(i).getStructureMap().get("医嘱开始时间");
            if (StringUtil.isNotBlank(startDateStr)) {
                if (i % 2 == 0) {
                    adviceDate = startDateStr + ",";
                } else {
                    adviceDate += startDateStr;
                    adviceEntry.add(adviceDate);
                }
            }
        }

        //所有查房记录的日期天
        List<Date> dateRecordDay = new ArrayList<>();

        String recordTime = "";
        List<ThreeLevelWardDoc> allDoctorWradDocs = inputInfo.getThreeLevelWardDocs().get(0).getAllDoctorWradDocs();//查房记录
        for (ThreeLevelWardDoc threeLevelWardDoc : allDoctorWradDocs) {
            Map<String, String> rescueStructureMap = threeLevelWardDoc.getStructureMap();
            recordTime = rescueStructureMap.get("查房日期");
            Date recordDate = StringUtil.parseDateTime(recordTime);
            if (recordDate == null) {
                continue;
            }
            dateRecordDay.add(DateUtil.dateZeroClear(recordDate));
        }

        //查房记录时间在医嘱下病重范围内
        List<Date> matchDates = new ArrayList<>();
        //危重患者应该有的查房记录的日期天
        Date criticallyStartDate = null, criticallyEndDate = null;
        for (String adviceDateStr : adviceEntry) {
            criticallyStartDate = StringUtil.parseDateTime(adviceDateStr.split(",")[0]);
            criticallyEndDate = StringUtil.parseDateTime(adviceDateStr.split(",")[1]);
            if (criticallyStartDate == null || criticallyEndDate == null) {
                continue;
            }
            criticallyStartDate = DateUtil.dateZeroClear(criticallyStartDate);
            criticallyEndDate = DateUtil.dateZeroClear(criticallyEndDate);
            for (Date dateRecord : dateRecordDay) {
                if ((dateRecord.after(criticallyStartDate) && dateRecord.before(criticallyEndDate))
                        || dateRecord.equals(criticallyStartDate) || dateRecord.equals(criticallyEndDate)) {
                    matchDates.add(dateRecord);
                }
            }

            for (int i = 0; i < matchDates.size(); i++) {
                if (i + 1 < matchDates.size()) {
                    if (CatalogueUtil.compareTime(matchDates.get(i), matchDates.get(i + 1), 24 * 60L)) {
                        status.set("-1");
                        return;
                    }
                }
            }
        }
    }
}