package com.lantone.qc.kernel.catalogue.medicalwriting;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.InformedConsentDoc;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.security.util.ListUtil;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @ClassName : MEDI03118
 * @Description : 知情同意书无患方签名或患方签名未授权
 * @Author : wsy
 * @Date: 2020-05-25 18:54
 */
@Component
public class MEDI03118 extends QCCatalogue {
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        List<InformedConsentDoc> informedConsentDocList = inputInfo.getInformedConsentDoc();
        if(ListUtil.isEmpty(informedConsentDocList)){
            return;
        }
        if (informedConsentDocList.size() == 0) {
            return;
        }

        long count = informedConsentDocList.stream().filter(i -> i.getStructureMap() != null && StringUtil.isNotBlank(i.getStructureMap().get("患方签名"))).count();
        if (count > 0 && count != informedConsentDocList.size()) {
            status.set("-1");
            return;
        }
    }
}