package com.lantone.qc.kernel.catalogue.operationdiscussion;

import com.alibaba.fastjson.JSONArray;
import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.DoctorAdviceDoc;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.model.doc.operation.*;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import java.text.ParseException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @Description: 手术记录次数与手术次数不一致
 * @author: cy
 * @time: 2021/04/15 10:01
 */
@Component
public class OPE03107 extends QCCatalogue {

    @Override
    protected void start(InputInfo inputInfo, OutputInfo outputInfo) throws ParseException {
        status.set("0");
        int opeCount=0;
        int opePreCount=0;
        int opeThrCount=0;
        int opeRecordCount=0;
        int opeDisCount=0;
        int opeFormCount=0;
        List<OperationDoc> operationDocs = inputInfo.getOperationDocs();
        List<DoctorAdviceDoc> doctorAdviceDocs = inputInfo.getDoctorAdviceDocs();
        List<String> listTime = new ArrayList<>();
        //手术记录次数
        if(ListUtil.isNotEmpty(operationDocs)){
            for (OperationDoc operationDoc : operationDocs) {
                OperationRecordDoc operationRecordDoc = operationDoc.getOperationRecordDoc();
                //手术记录次数
                if(operationRecordDoc!=null){
                    opeRecordCount++;
                }else {
                    return;
                }
                //术前讨论，小结次数
                PreoperativeDiscussionDoc preoperativeDiscussionDoc = operationDoc.getPreoperativeDiscussionDoc();
                if(preoperativeDiscussionDoc!=null){
                    opePreCount++;
                }
                //术后首程次数
                OperationDiscussionDoc operationDiscussionDoc = operationDoc.getOperationDiscussionDoc();
                if(operationDiscussionDoc!=null){
                    opeDisCount++;
                }
                //知情同意书次数
                OperationInformedConsentDoc operationInformedConsentDoc = operationDoc.getOperationInformedConsentDoc();
                if(operationInformedConsentDoc!=null){
                    opeFormCount++;
                }
            }
        }else {
            return;
        }
        //医嘱判断手术
        if (ListUtil.isNotEmpty(doctorAdviceDocs)) {
            for (DoctorAdviceDoc dad : doctorAdviceDocs) {
                Map<String, String> dadStructureMap = dad.getStructureMap();
                String daStatus = dadStructureMap.get(Content.doctorAdviceState);
                String adviceType = dadStructureMap.get(Content.doctorAdviceType);
                //取临时医嘱
                if (StringUtil.isNotEmpty(adviceType) && adviceType.equals(Content.statOrder)) {
                    if (StringUtil.isNotEmpty(daStatus)) {
                        if (!Content.cancellationOrderList.contains(daStatus)) {
                            String name = dadStructureMap.get("医嘱处方类型");
                            List<String> operationList = Content.operationList;
                            if (StringUtil.isNotEmpty(name)) {
                                if (operationList.contains(name)) {
                                    String daName = dadStructureMap.get(Content.medicalOrderName);
                                    Pattern p = Pattern.compile("[0-9]{1,2}[.|\\-|/][0-9]{1,2}日[0-9]{1,2}[:|\\-|/][0-9]{1,2}");
                                    Matcher matcher = p.matcher(daName);
                                    if (matcher.find()) {
                                        String group = matcher.group(0);
                                        if (!listTime.contains(group)) {
                                            listTime.add(group);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
        //查房
        List<ThreeLevelWardDoc> threeLevelWardDocs = inputInfo.getThreeLevelWardDocs();
        if(ListUtil.isNotEmpty(threeLevelWardDocs)){
            List<ThreeLevelWardDoc> allDoctorWradDocs = threeLevelWardDocs.get(0).getAllDoctorWradDocs();
            if(allDoctorWradDocs!=null){
                for (ThreeLevelWardDoc allDoctorWradDoc : allDoctorWradDocs) {
                    Map<String, String> structureMap = allDoctorWradDoc.getStructureMap();
                    String thrName = structureMap.get("查房标题");
                    if (StringUtil.isNotEmpty(thrName)) {
                        String substring = thrName.substring(0, 2);
                        if (substring.equals("术前")){
                            opeThrCount++;
                        }
                    }
                }
            }
        }

        //获取手术最多的次数
        opeCount=listTime.size();
        int max = opeCount>opePreCount?opeCount: opePreCount;
        max=max>opeThrCount?max:opeThrCount;
        max=max>opeDisCount?max:opeDisCount;
        max=max>opeFormCount?max:opeFormCount;
        if(max==opeRecordCount){
            return;
        }else{
            status.set("-1");
            return;
        }
    }
}