package com.lantone.qc.kernel.catalogue.threelevelward;

import com.google.common.collect.Lists;
import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.DoctorAdviceDoc;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.MapUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @ClassName : THR03115
 * @Description :  接受CT情况下病程记录缺少CT检查记录
 * @Author : zh
 * @Date: 2021-04-06 17:06
 */
@Component
public class THR03115 extends QCCatalogue {
    @Override
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        boolean flag = false;
        List<DoctorAdviceDoc> doctorAdviceDocsList = inputInfo.getDoctorAdviceDocs();
        if (ListUtil.isNotEmpty(doctorAdviceDocsList)) {
            for (DoctorAdviceDoc doctorAdviceDoc : doctorAdviceDocsList) {
                Map<String, String> doctorAdviceDocStructureMap = doctorAdviceDoc.getStructureMap();
                String daStatus = doctorAdviceDocStructureMap.get(Content.doctorAdviceState);
                if (StringUtil.isNotEmpty(daStatus)) {
                    if (!Content.cancellationOrderList.contains(daStatus)) {
                        //ct取临时医嘱
                        String doctorAsks = doctorAdviceDocStructureMap.get(Content.doctorAdviceType);
                        if (Content.statOrder.equals(doctorAsks)) {
                            //取医嘱项目名称
                            String daItemName = doctorAdviceDocStructureMap.get(Content.medicalOrderName);
                            String nameSs = doctorAdviceDocStructureMap.get("医嘱处方类型");
                            if (StringUtil.isNotEmpty(daItemName) && StringUtil.isNotEmpty(nameSs)) {
                                if (dateStr(daItemName) && (nameSs.equals("检查") || nameSs.equals("LAB"))) {
                                    flag = true;
                                    status.set("-1");
                                }
                            }
                        }
                    }
                }
            }
        }
        if (flag) {
            List<ThreeLevelWardDoc> threeLevelWardDocs = inputInfo.getThreeLevelWardDocs();
            if (ListUtil.isNotEmpty(threeLevelWardDocs)) {
                List<ThreeLevelWardDoc> allDoctorWradDocs = threeLevelWardDocs.get(0).getAllDoctorWradDocs();
                if (ListUtil.isNotEmpty(allDoctorWradDocs)) {
                    for (ThreeLevelWardDoc allDoctorWradDoc : allDoctorWradDocs) {
                        Map<String, String> structureMap = allDoctorWradDoc.getStructureMap();
                        String records = structureMap.get(Content.illnessRecords);
                        if (StringUtil.isNotEmpty(records)) {
                            if (dateStrCf(records)) {
                                status.set("0");
                                return;
                            }
                        }
                    }
                }
            }
        }
    }

    private boolean dateStr(String string) {
        if (StringUtil.isEmpty(string)) {
            return false;
        }
        String str = Str(string);
        String rex = "[\\s\\S]*(?=线断层)[^,;，；。]{0,8}(?=扫)[\\s\\S]*";
        String rex1 = "[\\s\\S]*(?=计算机)[^,;，；。]{0,8}(?=断层)[^,;，；。]{0,7}(?=扫)[\\s\\S]*";
        if (str.contains("CT") || str.matches(rex) || str.matches(rex1)) {
            if (str.contains("CT")) {
                List<String> notCTList = Content.notCTList;
                for (String list : notCTList) {
                    if (str.contains(list)) {
                        return false;
                    }
                }
            }
            return true;
        }
        return false;
    }

    private boolean dateStrCf(String string) {
        if (StringUtil.isEmpty(string)) {
            return false;
        }
        boolean ctCf = false;
        boolean jsj = false;
        ArrayList<String> cfStrings = new ArrayList<>();
        String str = Str(string);
        String rex = "[\\s\\S]*(?=线断层)[^,;，；。]{0,8}(?=扫)[\\s\\S]*";
        String rex1 = "[\\s\\S]*(?=计算机)[^,;，；。]{0,8}(?=断层)[^,;，；。]{0,7}(?=扫)[\\s\\S]*";
        String rex2 = "[\\s\\S]{0,5}(?=CT)[\\s\\S]{0,10}";
        //需要判断同时存在CT和OCT等情况的发生
        if (str.contains("CT")) {
            Matcher matcher = Pattern.compile(rex2).matcher(str);
            while (matcher.find()) {
                String group = matcher.group();
                cfStrings.add(group);
            }
            List<String> notCTList = Content.notCTList;
            if (ListUtil.isNotEmpty(cfStrings)) {
                for (String notCT : notCTList) {
                    for (int i = 0; i < cfStrings.size(); i++) {
                        String cfStr = cfStrings.get(i);
                        if (cfStr.contains(notCT)) {
                            cfStrings.remove(i);
                            continue;
                        }
                    }
                }
                if (cfStrings.size() > 0) {
                    ctCf = true;
                }
            }
        }
        if (str.matches(rex) || str.matches(rex1)) {
            jsj = true;
        }

        for (String word : words) {
            if (str.contains(word)) {
                return true;
            }
        }

        if (jsj || ctCf) {
            return true;
        }
        return false;
    }

    private String Str(String str) {
        str = str.replaceAll("\\*", "\\\\*");
        str = str.replaceAll("\\)", "\\\\)");
        str = str.replaceAll("\\.", "\\\\.");
        str = str.replaceAll("\\?", "\\\\?");
        str = str.replaceAll("\\+", "\\\\+");
        str = str.replaceAll("\\$", "\\\\$");
        str = str.replaceAll("\\^", "\\\\^");
        str = str.replaceAll("\\[", "\\\\[");
        str = str.replaceAll("\\]", "\\\\]");
        str = str.replaceAll("\\(", "\\\\(");
        str = str.replaceAll("\\{", "\\\\{");
        str = str.replaceAll("\\}", "\\\\}");
        str = str.replaceAll("\\|", "\\\\|");
        str = str.replaceAll("\\/", "\\\\/");
        return str;
    }

    List<String> words = Lists.newArrayList("胸部平扫");
}
