package com.lantone.qc.kernel.catalogue.threelevelward;

import com.lantone.qc.kernel.catalogue.QCCatalogue;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.model.InputInfo;
import com.lantone.qc.pub.model.OutputInfo;
import com.lantone.qc.pub.model.doc.DoctorAdviceDoc;
import com.lantone.qc.pub.model.doc.PacsDoc;
import com.lantone.qc.pub.model.doc.ThreeLevelWardDoc;
import com.lantone.qc.pub.util.ListUtil;
import com.lantone.qc.pub.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @ClassName : THR3113
 * @Description :  接受MR情况下病程记录缺少MR检查记录
 * @Author : zh
 * @Date: 2021-04-06 17:06
 */
@Component
public class THR3113 extends QCCatalogue {
    @Override
    public void start(InputInfo inputInfo, OutputInfo outputInfo) {
        status.set("0");
        boolean flag = false;
        List<DoctorAdviceDoc> doctorAdviceDocsList = inputInfo.getDoctorAdviceDocs();
        if (ListUtil.isNotEmpty(doctorAdviceDocsList)) {
            for (DoctorAdviceDoc doctorAdviceDoc : doctorAdviceDocsList) {
                Map<String, String> doctorAdviceDocStructureMap = doctorAdviceDoc.getStructureMap();
                String daStatus = doctorAdviceDocStructureMap.get(Content.doctorAdviceState);
                if (StringUtil.isNotEmpty(daStatus)) {
                    if (!Content.cancellationOrderList.contains(daStatus)) {
                        //ct取临时医嘱
                        String doctorAsks = doctorAdviceDocStructureMap.get(Content.doctorAdviceType);
                        if (Content.statOrder.equals(doctorAsks)) {
                            //取医嘱项目名称
                            String daItemName = doctorAdviceDocStructureMap.get(Content.medicalOrderName);
                            String nameSs = doctorAdviceDocStructureMap.get("医嘱处方类型");
                            if (StringUtil.isNotEmpty(daItemName) && StringUtil.isNotEmpty(nameSs)) {
                                if (dateStr(daItemName) && nameSs.equals("检查")) {
                                    flag = true;
                                    status.set("-1");
                                }
                            }
                        }
                    }
                }
            }
        }
        if(flag) {
            List<ThreeLevelWardDoc> threeLevelWardDocs = inputInfo.getThreeLevelWardDocs();
            if (ListUtil.isNotEmpty(threeLevelWardDocs)) {
                List<ThreeLevelWardDoc> allDoctorWradDocs = threeLevelWardDocs.get(0).getAllDoctorWradDocs();
                if (ListUtil.isNotEmpty(allDoctorWradDocs)) {
                    for (ThreeLevelWardDoc allDoctorWradDoc : allDoctorWradDocs) {
                        Map<String, String> structureMap = allDoctorWradDoc.getStructureMap();
                        String records = structureMap.get(Content.illnessRecords);
                        if (StringUtil.isNotEmpty(records)) {
                            if (dateStrBg(records)) {
                                status.set("0");
                                return;
                            }
                        }
                    }
                }
            }
        }
    }
    private boolean dateStr(String str){
        if(StringUtil.isEmpty(str)){
            return false;
        }
        if (str.contains("MR") || str.contains("磁共振")||str.contains("核磁")) {
            List<String> notMRList = Content.notMRList;
            for (String list : notMRList) {
                if(!str.contains(list)){
                    return true;
                }
            }
        }
        return false;
    }
    private boolean dateStrBg(String str){
        if(StringUtil.isEmpty(str)){
            return false;
        }
        boolean MrCf=false;
        boolean jsj=false;
        ArrayList<String> cfStrings = new ArrayList<>();
        String rex2="[\\s\\S]{0,5}(?=MR)[\\s\\S]{1,10}";
        //需要判断同时存在MR和MRS等情况的发生
        if(str.contains("MR")) {
            Matcher matcher = Pattern.compile(rex2).matcher(str);
            while (matcher.find()) {
                String group = matcher.group();
                cfStrings.add(group);
            }
            List<String> notMRList = Content.notMRList;
            if (ListUtil.isNotEmpty(cfStrings)) {
                for (String notCT : notMRList) {
                    for (int i = 0; i < cfStrings.size(); i++) {
                        String cfStr = cfStrings.get(i);
                        if(cfStr.contains(notCT)){
                            cfStrings.remove(i);
                            continue;
                        }
                    }
                }
                if(cfStrings.size()>0){
                    MrCf=true;
                }
            }

        }
        if ( str.contains("磁共振")||str.contains("核磁")) {
            jsj= true;
        }
        if(jsj||MrCf){
            return true;
        }
        return false;
    }

}
