package com.lantone.qc.trans.beilun.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.pub.Content;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.util.List;
import java.util.Map;

/**
 * @Description: 手术知情同意书html解析
 * @author: HUJING
 * @time: 2020/9/14 15:41
 */
@Slf4j
public class BeiLunOperationInformedHtmlAnalysis implements BeiLunHtmlAnalysis {

    private String modeId = "16";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> structureMap = Maps.newLinkedHashMap();
        try {
            List<String> titles = Lists.newArrayList(
                    "术种使用的植入性材料或高值耗材及风险",
                    "医师陈述",
                    "患方签名",
                    "签字时间",
                    "签名时间");
            String html = args[0];
            String recTitle = args[1];
            String recTypeId = args[2];
            Document doc = Jsoup.parse(html);
            String htmlContent = BeiLunHtmlAnalysisUtil.blockDivToStr(doc.selectFirst("body").child(0), false);
            if (StringUtil.isNotBlank(htmlContent)) {
                htmlContent = htmlContent.replaceAll("[   ]", " ");
                CommonAnalysisUtil.html2StructureMap(titles, htmlContent, structureMap);
            }
            CommonAnalysisUtil.extractDateByTitle(structureMap, "记录时间");
            for (String ignoreContrast : ignoreContrasts) {
                if(structureMap.containsKey( ignoreContrast)){
                    structureMap.remove(ignoreContrast);
                }
            }

            resultHandle(structureMap);
            parseMethod(structureMap);
            BeiLunHtmlAnalysisUtil.insertModuleId(modeId, recTypeId, structureMap);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return structureMap;
    }
    private List<String>  ignoreContrasts = Lists.newArrayList(
            "医师陈述",
            "患方签名",
            "签字时间",
            "签名时间"
    );

    private void parseMethod(Map<String, String> result) {
        if (result.containsKey("术种使用的植入性材料或高值耗材及风险") && StringUtils.isNotEmpty(result.get("术种使用的植入性材料或高值耗材及风险"))) {
            Boolean flag= true;
            String str = result.get("术种使用的植入性材料或高值耗材及风险");
            if(str.contains("\uF0FE无") && str.contains("\uF06F有")){
                result.put("术种使用的植入性材料或高值耗材及风险","false");
                flag = false;
            }
            if(str.contains("\uF06F无") && str.contains("\uF0FE有")){
             /*   str = str.split("\uF0FE有")[1];
                str = str.replace(",","，").replace("。","，").replace("   ","，");
                str = str.split("，")[0];
                result.put("植入性材料",str);*/
                result.put("术种使用的植入性材料或高值耗材及风险","true");
                Boolean modelFlag = false;
                for (String model : Content.MODELIST) {
                    if(str.contains(model)){
                        modelFlag = true;
                        break;
                    }
                }
                Boolean numFlag = false;
                for (String number : Content.NUMBERLIST) {
                    if(str.contains(number)){
                        numFlag = true;
                        break;
                    }
                }
                if(modelFlag && numFlag){
                    result.put("型号和数量符合要求","true");
                }

                flag = false;
            }
            if(flag){
                result.put("术种使用的植入性材料或高值耗材及风险","");
            }

        }

}

    private void resultHandle(Map<String, String> result) {
        if (MapUtils.isEmpty(result)) {
            return;
        }
        result.forEach((key, value) -> {
            if (StringUtil.isNotBlank(value)) {
                value = StringUtil.trim(value);
                if (value.endsWith("；") || value.endsWith(";")) {
                    value = value.substring(0, value.length() - 1);
                }
                result.put(key, value);
            }
        });
    }
}