package com.lantone.qc.trans.ningbozhenhai;

import com.google.common.collect.Lists;
import com.lantone.qc.pub.model.doc.CriticallyIllNoticeDoc;
import com.lantone.qc.pub.model.vo.MedrecVo;
import com.lantone.qc.pub.util.FastJsonUtils;
import com.lantone.qc.trans.ModelDocTrans;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import com.lantone.qc.trans.comsis.ModelDocGenerate;
import com.lantone.qc.trans.comsis.OrdinaryAssistant;
import com.lantone.qc.trans.ningbozhenhai.util.CriticallyIllNoticeHtmlAnalysis;
import com.lantone.qc.trans.ningbozhenhai.util.HtmlAnalysis;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;

import java.util.List;
import java.util.Map;

/**
 * @Description: 病危通知书文档生成
 * @author: rengb
 * @time: 2020/3/19 19:41
 */
@Slf4j
public class CriticallyIllNoticeDocTrans extends ModelDocTrans {

    @Override
    public List<CriticallyIllNoticeDoc> extract(MedrecVo medrecVo) {
        List<CriticallyIllNoticeDoc> retList = Lists.newArrayList();
        List<Map<String, Object>> contentMaps = (List) medrecVo.getContent().get("content");
        contentMaps.forEach(contentMap -> {
            try {
                CriticallyIllNoticeDoc criticallyIllNoticeDoc = getCriticallyIllNoticeDoc(contentMap);
                if (criticallyIllNoticeDoc != null) {
                    retList.add(criticallyIllNoticeDoc);
                }
            } catch (Exception e) {
                log.error(e.getMessage(), e);
            }
        });
        return retList;
    }

    private CriticallyIllNoticeDoc getCriticallyIllNoticeDoc(Map<String, Object> contentMap) {
        String content = contentMap.get("xmlText").toString();
        Map<String, String> structureMap = null;
        if (contentMap.get("isParsed") != null && "1".equals(contentMap.get("isParsed").toString())) {
            structureMap = (Map) FastJsonUtils.getJsonToMap(content);
        } else {
            String recTitle = contentMap.get("recTitle").toString();
            String recTypeId = contentMap.get("recTypeId").toString();
            HtmlAnalysis ningBoZhongYiHtmlAnalysis = new CriticallyIllNoticeHtmlAnalysis();
            Map<String, String> sourceMap = ningBoZhongYiHtmlAnalysis.analysis(content, recTitle, recTypeId);
            if (MapUtils.isNotEmpty(sourceMap)) {
                structureMap = OrdinaryAssistant.mapKeyContrast(sourceMap, keyContrasts);
                structureMap.put("记录编号", contentMap.get("recId").toString());
                structureMap.put("病历号", contentMap.get("behospitalCode") == null ? null : contentMap.get("behospitalCode").toString());
            }
        }
        if (MapUtils.isNotEmpty(structureMap)) {
            CriticallyIllNoticeDoc criticallyIllNoticeDoc = ModelDocGenerate.criticallyIllNoticeDocGen(structureMap);
            criticallyIllNoticeDoc.setText(CommonAnalysisUtil.html2String(content));
            criticallyIllNoticeDoc.setPageData((Map) structureMap);
            return criticallyIllNoticeDoc;
        } else {
            return null;
        }
    }

    private List<String> keyContrasts = Lists.newArrayList(
            "1.简要病情=简要病情",
            "2.目前诊断=目前诊断",
            "3.目前病情评估=目前病情评估",
            "4.目前患者病情危重，随时出现以下一种或多种危及患者生命的并发症，且病情有进一步恶化可能=进一步恶化可能",
            "5.注意事项=注意事项",
            "6.患方知情选择=患方知情选择"
    );

}