package com.lantone.qc.trans.ningbozhenhai.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: HUJING
 * @time: 2020/9/9 11:24
 */
@Slf4j
public class FirstCourseRecordHtmlAnalysis implements HtmlAnalysis {

    private String modeId = "2";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> structureMap = Maps.newLinkedHashMap();
        try {
            List<String> titles = Lists.newArrayList("病例特点", "拟诊讨论", "中医辨病辨证依据", "中医鉴别诊断", "西医鉴别诊断",
                    "鉴别诊断", "西医诊断依据", "诊断依据", "初步诊断", "病情评估", "诊疗计划", "医生签名");
            String html = args[0];
            String recTitle = args[1];
            Document doc = Jsoup.parse(html);
            String htmlContent = HtmlAnalysisUtil.blockDivToStr(doc.selectFirst("body").child(0), true)
                    .replace("第1页", "");
            CommonAnalysisUtil.extractWardInfo(recTitle, htmlContent, structureMap);
            if (StringUtil.isNotBlank(htmlContent)) {
                titles = CommonAnalysisUtil.sortTitles(titles, htmlContent);
                CommonAnalysisUtil.cutByTitles(htmlContent, titles, 0, structureMap);
                if (!structureMap.containsKey("诊断依据")) {
                    String zdyj = "";
                    if (structureMap.containsKey("西医诊断依据")) {
                        zdyj += "西医诊断依据：" + structureMap.get("西医诊断依据");
                    }
                    if (structureMap.containsKey("中医辨病辨证依据")) {
                        zdyj += "  中医辨病辨证依据：" + structureMap.get("中医辨病辨证依据");
                    }
                    structureMap.put("诊断依据", zdyj);
                }
                if (!structureMap.containsKey("鉴别诊断")) {
                    String jbzd = "";
                    if (structureMap.containsKey("西医鉴别诊断")) {
                        jbzd += "西医鉴别诊断：" + structureMap.get("西医鉴别诊断");
                    }
                    if (structureMap.containsKey("中医鉴别诊断")) {
                        jbzd += "  中医鉴别诊断：" + structureMap.get("中医鉴别诊断");
                    }
                    structureMap.put("鉴别诊断", jbzd);
                }

                /*if (structureMap.containsKey("诊疗计划")) {
                    if (structureMap.get("诊疗计划").indexOf("中医治则") != -1) {
                        structureMap.put("诊疗计划", structureMap.get("诊疗计划").substring(0, structureMap.get("诊疗计划").indexOf("中医治则")));
                    }
                }*/
                if (structureMap.containsKey("医生签名")) {
                    if (structureMap.get("医生签名").indexOf("SignControl") != -1) {
                        structureMap.put("医生签名", structureMap.get("医生签名").substring(0, structureMap.get("医生签名").indexOf("SignControl")));
                    }
                }
            }

            HtmlAnalysisUtil.insertModuleId(modeId, recTitle, structureMap);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return structureMap;
    }

}