package com.lantone.qc.trans.ningbozhenhai.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: HUJING
 * @time: 2020/9/9 11:24
 */
@Slf4j
public class LeaveHospitalHtmlAnalysis implements HtmlAnalysis {

    private String modeId = "5";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> structureMap = Maps.newLinkedHashMap();
        try {
            List<String> titles = Lists.newArrayList("姓名", "性别", "年龄 ", "科室", "床号", "住院号", "地址 ",
                    "入院时间", "出院日期", "入院诊断", "出院诊断", "入院情况", "辅助检查", "诊治经过", "住院重要检查化验结果",
                    "出院情况", "治疗效果", "出院医嘱", "特殊检查资料号", "医生签名", "日期", "专家门诊时间", "专科门诊时间", "健康宣教",
                    "健康指导");
            String html = args[0];
            String recTitle = args[1];
            String recTypeId = args[2];
            Document doc = Jsoup.parse(html);
            HtmlAnalysisUtil.tableStyle1InsertMap(doc.getElementById("table1"), structureMap);
            doc.selectFirst("body").child(0).getElementById("table1").remove();
            String htmlContent = HtmlAnalysisUtil.blockDivToStr(doc.selectFirst("body").child(0), true);
            if (StringUtil.isNotBlank(htmlContent)) {
                htmlContent = htmlContent.replace("患者如有需要，可于出院十日后（工作日），携带（身份证、社保卡、户口簿）等有效身份证到病案室（茗\n" +
                        "园楼3楼）复印住院病历等资料。如委托他人办理，需携带委托人（患者）及被委托人有效身份证明前来办\n" +
                        "理。", "")
                        .replace("患者如有需要，可于出院十日后，携带（身份证、社保卡、户口簿等）有效身份证明到病案室（茗园楼\n" +
                                "三楼）复印住院病历资料。如委托他人办理，需携带委托人（患者）及被委托人有效身份证明前来办理。", "")
                        .replace("特殊检查资料号", "特殊检查资料号：");
                List<String> sortTitles = CommonAnalysisUtil.sortTitles(titles, htmlContent);
                CommonAnalysisUtil.cutByTitles(htmlContent, sortTitles, 0, structureMap);
            }
            if (StringUtil.isNotBlank(CommonAnalysisUtil.extractDate(structureMap.get("出院日期")))) {
                structureMap.put("住院天数", structureMap.get("出院日期").replace(CommonAnalysisUtil.extractDate(structureMap.get("出院日期")), ""));
                structureMap.put("出院日期", CommonAnalysisUtil.extractDate(structureMap.get("出院日期")));
            }
            if (structureMap.containsKey("出院诊断") && structureMap.get("出院诊断").contains("入院情况")) {
                String discharge = structureMap.get("出院诊断");
                int index = discharge.indexOf("入院情况");
                structureMap.put("出院诊断", discharge.substring(0, index));
                structureMap.put("入院情况", discharge.substring(index + 4));
            }
            if (structureMap.containsKey("医生签名")) {
                if (structureMap.get("医生签名").indexOf("SignControl") != -1) {
                    structureMap.put("医生签名", structureMap.get("医生签名").substring(0, structureMap.get("医生签名").indexOf("SignControl")));
                }
            }
            if (structureMap.containsKey("日期")) {
                structureMap.put("日期", CommonAnalysisUtil.extractDate(structureMap.get("日期")));
            }
            HtmlAnalysisUtil.insertModuleId(modeId, recTypeId, structureMap);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return structureMap;
    }
}