package com.lantone.qc.trans.ningbozhenhai.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: rengb
 * @time: 2020/9/28 16:36
 */
@Slf4j
public class OperationRecordHtmlAnalysis implements HtmlAnalysis {

    private String modeId = "17";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> map = Maps.newLinkedHashMap();
        try {
            String html = args[0];
            String recTitle = args[1];
            String recTypeId = args[2];
            Element bigDivElement = Jsoup.parse(html).selectFirst("body").child(0);
            analysisGeneral(bigDivElement, map);
            HtmlAnalysisUtil.mapValueRemoveStartColon(map);
            CommonAnalysisUtil.makeEmpty(map, "医生签名");
            HtmlAnalysisUtil.insertModuleId(modeId, recTypeId, map);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return map;
    }

    private void analysisGeneral(Element bigDivElement, Map<String, String> map) {
        String text = HtmlAnalysisUtil.blockDivToStr(bigDivElement, true);
        text = text.replaceAll("第1页", "")
                .replace("（包括体位、切口处理经过，病灶所见及手术步骤等）：", "");
        List<String> titles = Lists.newArrayList(
                "姓名", "病区", "科室", "床号", "床位", "住院号", "手术开始时间", "手术结束时间", "术前诊断", "术 前 诊  断",
                "手术名称", "手 术 名  称", "术中诊断", "术(中)后诊断", "手术人员", "手术主刀医师", "手 术 主 刀 医 师", "一 助", "二 助", "麻醉方式",
                "麻 醉 方  式", "麻醉人员", "麻 醉 医  师", "手术风险评估（可选）", "NNIS分级", "手术类别（可选）", "手术标本", "冰冻切片结果",
                "术中并发症", "术中失血量", "手术经过", "手术经过及处理（包括患者的体位、切口处理、病灶所见及手术步骤等）", "记录者签字", "主刀签字", "主刀医师签名",
                "记录日期", "日期", "时间"
        );
        titles = CommonAnalysisUtil.sortTitles(titles, text);
        CommonAnalysisUtil.cutByTitles(text, titles, 0, map);
        CommonAnalysisUtil.processTypeRight(map, "NNIS分级");
        CommonAnalysisUtil.processTypeRight(map, "手术类别（可选）");
        CommonAnalysisUtil.processType(map, "手术标本");
        CommonAnalysisUtil.processType(map, "术中并发症");
        map.put("NNIS分级", map.get("NNIS分级").replace("-", ""));
        if (map.containsKey("时间")) {
            map.put("日期", map.get("记录日期") + map.get("时间"));
        }
        if (map.containsKey("手术主刀医师")) {
            map.put("手术人员", "手术主刀医师：" + map.get("手术主刀医师") + " 一助：" + map.get("一助") + " 二助：" + map.get("二助"));
        }
    }

}