package com.lantone.qc.trans.shengzhouyy;

import com.google.common.collect.Lists;
import com.lantone.qc.dbanaly.facade.yiwu.YiWuXmlUtil;
import com.lantone.qc.dbanaly.util.ModuleMappingUtil;
import com.lantone.qc.pub.model.doc.ClinicalBloodDoc;
import com.lantone.qc.pub.model.vo.MedrecVo;
import com.lantone.qc.pub.util.FastJsonUtils;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.ModelDocTrans;
import com.lantone.qc.trans.comsis.OrdinaryAssistant;
import org.apache.commons.collections.MapUtils;

import java.util.List;
import java.util.Map;

public class ClinicalBloodDocTrans extends ModelDocTrans {
    private String modeId = "10";

    @Override
    public List<ClinicalBloodDoc> extract(MedrecVo medrecVo) {
        List<ClinicalBloodDoc> retList = Lists.newArrayList();
        List<Map<String, Object>> contentMaps = (List) medrecVo.getContent().get("content");
        contentMaps.forEach(contentMap -> {
            retList.add(getClinicalBloodDoc(contentMap));
        });
        return retList;
    }

    private ClinicalBloodDoc getClinicalBloodDoc(Map<String, Object> contentMap) {
        String content = contentMap.get("xmlText").toString();
        Map<String, String> structureMap = null;
        if (contentMap.get("isParsed") != null && "1".equals(contentMap.get("isParsed").toString())) {
            structureMap = (Map) FastJsonUtils.getJsonToMap(content);
        } else {
            Map<String, String> sourceMap = YiWuXmlUtil.xmlToMap(content);
            sourceMap.put("mode_id", ModuleMappingUtil.getDeptModuleId(modeId));
            sourceMap.put("mode_id=" + contentMap.get("modeId").toString(), "");
            sourceMap.put("rec_title=" + contentMap.get("recTitle").toString(), "");
            structureMap = OrdinaryAssistant.mapKeyContrast(sourceMap, keyContrasts, modeId);
        }
        if (MapUtils.isNotEmpty(structureMap)) {
            String CliBDateStr = structureMap.get("输血日期");
            if (StringUtil.isNotBlank(CliBDateStr) && CliBDateStr.length() == 39) {
                structureMap.put("输注开始", CliBDateStr.substring(0, 19));
                structureMap.put("输注结束", CliBDateStr.substring(20));
            }
        }

        ClinicalBloodDoc clinicalBloodDoc = new ClinicalBloodDoc();
        clinicalBloodDoc.setStructureMap(structureMap);
        clinicalBloodDoc.setPageData((Map) structureMap);
        return clinicalBloodDoc;
    }

    private List<String> keyContrasts = Lists.newArrayList(
            /*"开始时间=输注开始",
            "医生=记录医生",
            "记录医师=记录医生",
            "结束时间=输注结束",
            "内容=输注后效果评价",
            "输血原因（可多选）=输注原因",
            "ABO=血型",
            "医师签名=记录医生",
            "记录医师签名=记录医生",
            "病程时间=记录时间",
            "病情内容=病历内容",
            "病程内容=病历内容"*/
            "病程时间=病历日期",
            "病程标题=标题",
            "病程内容=病历内容",
            "记录医师签名=记录医生",
            "病程时间=记录时间"
    );
}
