package com.lantone.qc.trans.wenfuyi.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.util.List;
import java.util.Map;

/**
 * @Description: 疑难、危重病例讨论记录html解析
 * @author: HUJING
 * @time: 2020/9/14 15:41
 */
@Slf4j
public class DifficultCaseDiscussHtmlAnalysis implements WenFuYiHtmlAnalysis {

    private String modeId = "19";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> structureMap = Maps.newLinkedHashMap();
        try {
            List<String> titles = Lists.newArrayList("病例类别", "文书类别", "讨论时间", "讨论地点", "主持人", "病例报告人",
                    "参加人员", "临床诊断", "汇报病史", "具体讨论意见", "主持人小结意见", "主持人(科主任或科主任授权的科室副主任)签名", "记录医师", "记录日期");
            String html = args[0];
            String recTitle = args[1];
            String recTypeId = args[2];
            Document doc = Jsoup.parse(html);
            String htmlContent = doc.selectFirst("body").children().text();
            if (StringUtil.isNotBlank(htmlContent)) {
                CommonAnalysisUtil.extractWardInfo(recTitle,htmlContent, structureMap);
                CommonAnalysisUtil.html2StructureMap(titles, htmlContent, structureMap);
            }
            CommonAnalysisUtil.extractDateByTitle(structureMap, "告知时间");
            WenFuYiHtmlAnalysisUtil.insertModuleId(modeId, recTypeId, structureMap);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return structureMap;
    }

    /**
     * 去除value中最后的逗号
     *
     * @param structureMap
     */
    private void removeLast(Map<String, String> structureMap) {
        for (Map.Entry<String, String> entry : structureMap.entrySet()) {
            structureMap.put(entry.getKey(), findLast(entry.getValue()));
        }
    }

    private String findLast(String value) {
        if (StringUtil.isNotBlank(value)) {
            if (value.lastIndexOf("，") != value.length() - 1) {
                return value;
            }
            value = value.substring(0, value.length() - 1);
            findLast(value);
        }
        return value;
    }
}