package com.lantone.qc.trans.wenfuyi.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.pub.util.StringUtil;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: HUJING
 * @time: 2020/9/9 11:24
 */
@Slf4j
public class LeaveHospitalHtmlAnalysis implements WenFuYiHtmlAnalysis {

    private String modeId = "5";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> structureMap = Maps.newLinkedHashMap();
        try {
            List<String> titles = Lists.newArrayList(
                    "入院诊断(Admitting diagnosis)", "出院诊断(Discharge diagnosis)", "入院情况(Admission status)", "诊疗经过(Hospital course)", "出院情况(Discharge status)", "疗效评定(Curative effect)",
                    "出院医嘱(Discharge instructions)", "患者/法定监护人/委托代理人签名", "签名时间");
            String html = args[0];
            String recTitle = args[1];
            String recTypeId = args[2];
            Document doc = Jsoup.parse(html);
            String htmlContent = WenFuYiHtmlAnalysisUtil.blockDivToStr(doc.selectFirst("body"), true);
            if (StringUtil.isNotBlank(htmlContent)) {
                htmlContent = htmlContent.replace("医务人员已告知出院录中主要内容。", "")
                        .replace("诊疗经过(Hospital course)", "诊疗经过(Hospital course)：")
                        .replace("出院医嘱(Discharge instructions)", "出院医嘱(Discharge instructions)：");
                List<String> sortTitles = CommonAnalysisUtil.sortTitles(titles, htmlContent);
                CommonAnalysisUtil.cutByTitles(htmlContent, sortTitles, 0, structureMap);
            }
            if (structureMap.get("签名时间") != null) {
                structureMap.put("签名时间", structureMap.get("签名时间").replace("年 月 日 时 分", ""));
            }
            WenFuYiHtmlAnalysisUtil.insertModuleId(modeId, recTypeId, structureMap);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return structureMap;
    }
}