package com.lantone.qc.trans.wenfuyi.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: rengb
 * @time: 2020/9/28 14:56
 */
@Slf4j
public class OperationHtmlAnalysis implements WenFuYiHtmlAnalysis {

    private String modeId = "18";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> map = Maps.newLinkedHashMap();
        try {
            String html = args[0];
            String recTitle = args[1];
            String recTypeId = args[2];
            String bigDivElement = Jsoup.parse(html).selectFirst("body").children().text();
            analysisGeneral(recTitle, bigDivElement, map);
            CommonAnalysisUtil.makeEmpty(map, "医生签名");
            WenFuYiHtmlAnalysisUtil.insertModuleId(modeId, recTypeId, map);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return map;
    }

    private void analysisGeneral(String recTitle, String bigDivElement, Map<String, String> map) {
        String text = bigDivElement.replace("温馨提示：【术后首次病程记录】请在离开手术室之前书写完毕，返回病房后及时告知患方并签字，签字时间应迟于记录时间。", "")
                .replace("温馨提示：【术后首次病程记录】请先在手术室录入，然后回病房打印。", "")
                .replace("注：病灶描述。", "").replace("注：手术简要步骤、病灶切除、送病理等。", "")
                .replace("注：病人回病房时的一般情况，如生命体征、麻醉后情况等。", "").replace("注：包括用药、引流管等。", "");

        int index1 = text.indexOf("主刀/I助签名");
        int index2 = text.indexOf("患者/法定监护人/委托代理人签名");
        if (!text.contains("记录时间")) {
            if (index1 != -1 && index2 != -1) {
                text = text.substring(0, index1)
                        + text.substring(index1, index2).replace("签名时间", "主刀签字时间").replace("年月日时分秒", "")
                        + text.substring(index2).replace("签名时间", "患方签字时间").replace("年月日时分秒", "");
            }
        }

        CommonAnalysisUtil.extractWardInfo(recTitle, text, map);
        List<String> titles = Lists.newArrayList(
                "手术开始时间", "手术结束时间", "手术时间", "麻醉方式", "手术方式", "术中诊断", "术中所见", "手术简要经过", "患者术后情况", "术后处理措施", "术后应当特别注意观察的事项",
                        "主刀/I助签名", "患者/法定监护人/委托代理人签名", "主刀签字时间", "患方签字时间", "记录时间"
        );
        titles = CommonAnalysisUtil.sortTitles(titles, text);
        CommonAnalysisUtil.cutByTitles(text, titles, 0, map);
    }

}