package com.lantone.qc.trans.wenfuyi.util;

import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.lantone.qc.trans.comsis.CommonAnalysisUtil;
import lombok.extern.slf4j.Slf4j;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Element;

import java.util.List;
import java.util.Map;

/**
 * @Description:
 * @author: rengb
 * @time: 2020/9/28 20:12
 */
@Slf4j
public class RescueHtmlAnalysis implements WenFuYiHtmlAnalysis {

    private String modeId = "22";

    @Override
    public Map<String, String> analysis(String... args) {
        Map<String, String> map = Maps.newLinkedHashMap();
        try {
            String html = args[0];
            String recTitle = args[1];
            String recTypeId = args[2];
            Element bigDivElement = Jsoup.parse(html).selectFirst("body");
            analysisGeneral(recTitle, bigDivElement, map);
            WenFuYiHtmlAnalysisUtil.insertModuleId(modeId, recTypeId, map);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
        }
        return map;
    }

    private void analysisGeneral(String recTitle, Element bigDivElement, Map<String, String> map) {
        String text = WenFuYiHtmlAnalysisUtil.blockDivToStr(bigDivElement, true).replace("备注：如抢救失败，应动员近亲属做尸检，签署尸检同意书。如近亲属拒绝尸检及拒签同意书应记录在病历上", "");
        CommonAnalysisUtil.extractWardInfo(recTitle, text, map);
        List<String> titles = Lists.newArrayList(
                "抢救时间", "参加抢救人员姓名和职称", "病情变化情况", "抢救措施及详细经过"
        );
        titles = CommonAnalysisUtil.sortTitlesNoColon(titles, text);
        CommonAnalysisUtil.cutByTitlesNoColon(text, titles, 0, map);
    }

}